#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


import utils, os.path, re, types
import indexer, server, providers

ATTR_INDEXED = 'indexed'
ATTR_DIRTY = 'dirty'

class Book:
    def __init__(self, title, provider, attrs):
        """Initializes book record. 'provider' is string identifying data
           provider plugin, 'attrs' is dictionary of provider-specific
           settings such as filename or source URL."""
        self.title = title
        self.provider = provider
        self.attr = attrs
        if ATTR_INDEXED not in self.attr:
            self.attr[ATTR_INDEXED] = '0'
        if ATTR_DIRTY not in self.attr:
            self.attr[ATTR_DIRTY] = '1'
        self.bookmarks = {}

    def save(self, cfg):
        """Saves the book into wxConfig object cfg."""
        key = self.title.replace('/','_').replace('\\','_').\
                         replace(':','_').replace(' ','_')
    
        allBooks = cfg.Read('/Books/list', '').split(':')
        if key not in allBooks:
            allBooks.append(key)
            cfg.Write('/Books/list', ':'.join(allBooks))
        
        cfg.SetPath('/Books/%s' % key)
        cfg.Write('title', self.title)
        cfg.Write('provider', self.provider)
        for i in range(0, len(self.bookmarks.keys())):
            key = self.bookmarks.keys()[i]
            cfg.Write('bookmarks/%i/title' % i, key)
            cfg.Write('bookmarks/%i/url' % i, self.bookmarks[key])
        for a in self.attr:
            cfg.Write('attr/%s' % a, self.attr[a])
        cfg.SetPath('/')

    def getProviderObj(self):
        return providers.providers[self.provider]

    def setAttr(self, attr, value):
        modified = ((attr not in self.attr) or (self.attr[attr] != value))
        if modified:
            self.attr[attr] = value
            if attr != ATTR_DIRTY:
                self.attr[ATTR_DIRTY] = '1'

    def getAttr(self, attr):
        return self.attr[attr]

    def hasAttr(self, attr):        
        return attr in self.attr

    def addBookmark(self, name, url):
        # remove http://localhost:XXXX/ prefix, so that the bookmark
        # is independent of used port number:
        url2 = re.sub("^http://localhost:%i" % utils.getServerPort(), "", url)
        self.bookmarks[name] = url2

    def delBookmark(self, name):
        del self.bookmarks[name]

    def getBookmarks(self):
        """Returns tuple (keys,dict), where 'keys' is sorted list of bookmark
           titles and 'dict' is dictionary with URLs, keyed by the titles."""
        port = utils.getServerPort()
        keys = self.bookmarks.keys()
        keys.sort()
        out = {}
        for k in keys:
            out[k] = "http://localhost:%i%s" % (port, self.bookmarks[k])
        return (keys, out)

    def getHomeURL(self):
        """Returns URL that can be passed to the browser for root page."""
        url = "http://localhost:%i/%s/%s" % \
              (utils.getServerPort(), utils.mangleBookName(self.title),
              self.getProviderObj().getHomeURL(self))
        return url
        
    
def loadBook(cfg):
    title = cfg.Read('title')
    provider = cfg.Read('provider')
    attr = {}
    bookmarks = {}
    cfg.SetPath('attr')
    cont, name, index = cfg.GetFirstEntry()
    while cont:
        attr[name] = cfg.Read(name)
        cont, name, index = cfg.GetNextEntry(index)
    book = Book(title, provider, attr)
    cfg.SetPath('../bookmarks')
    cont, name, index = cfg.GetFirstGroup()
    while cont:
        book.bookmarks[cfg.Read('%s/title' % name)] = cfg.Read('%s/url' % name)
        cont, name, index = cfg.GetNextGroup(index)
    return book



books = {}

def loadBooks():
    global books

    cfg = utils.config()
    cfg.SetPath('/Books')
    books = {}
    allBooks = cfg.Read('/Books/list').split(':')
    for name in allBooks:
        if name == '': continue
        cfg.SetPath('/Books/%s' % name)
        book = loadBook(cfg)
        books[book.title] = book
    cfg.SetPath('/')

def saveBooks():
    cfg = utils.config()
    cfg.SetPath('/')
    cfg.RenameGroup('Books', '__delete_Books')
    for b in books.values():
        b.save(cfg)
    cfg.DeleteGroup('__delete_Books')

def setBooks(newBooks):
    global books
    server.serverLock.acquire()
    for b in books:
        if b not in newBooks:
            indexer.disableIndex(books[b])
    books = newBooks
    server.serverLock.release()
